@@ -6,32 +6,31 @@ set -e
 ## label is set by jenkins
 #export label=${label}
 
-export GEOS_VER=3.9.6
-export WORKSPACE=/home/jenkins/workspace
-export GEOS_PATH=${WORKSPACE}/geos/rel-${GEOS_VER}
+SCRIPT=$(readlink -f "$0")
+export CUR_DIR=$(dirname "$SCRIPT")
+echo $CUR_DIR
+export CONFIG_FILE="$CUR_DIR/configs.sh"
+. $CONFIG_FILE
+echo $PATH
 
 cd ${WORKSPACE}/PostGIS_Worker_Run/label/${label}/$BRANCH
-export OS_BUILD=64
-export PG_VER=13
-export PGPATH=${WORKSPACE}/pg/label/${label}/rel/pg${PG_VER}w${OS_BUILD}
-
-export PATH=${PATH}:${PGPATH}/bin:${PGPATH}/lib
-export PGPORT=55432
-export PGDATA=$PGPATH/data_${PGPORT}
-export PGHOST=localhost
 
 sh autogen.sh
-./configure --with-pgconfig=${PGPATH}/bin/pg_config --with-library-minor-version
+./configure --with-pgconfig=${PGPATH}/bin/pg_config --with-geosconfig=${GEOS_PATH}/bin/geos-config --with-library-minor-version
 #make clean
 make
 export err_status=0
 make check RUNTESTFLAGS="-v"
 make install
+
 make check RUNTESTFLAGS="-v --extension"
 err_status=$?
+
+
 make garden
 err_status=$?
 
+
 if [ -d $PGDATA/postmaster.pid ] ; then
 	$PGCTL stop -D $PGDATA -s -m fast
 fi
