@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 set -e
 ## begin variables passed in by jenkins
 
@@ -65,7 +65,7 @@ fi
     --with-pgconfig=${PROJECTS}/pg/rel/pg${PG_VER}w${OS_BUILD}/bin/pg_config \
     --with-geosconfig=${PROJECTS}/geos/rel-${GEOS_VER}w${OS_BUILD}/bin/geos-config \
     --with-gdalconfig=${PROJECTS}/gdal/rel-${GDAL_VER}w${OS_BUILD}/bin/gdal-config \
-    --with-library-minor-version --without-interrupt-tests \
+    --without-interrupt-tests \
     --prefix=${PROJECTS}/pg/rel/pg${PG_VER}w${OS_BUILD}
 make clean
 make
@@ -118,8 +118,10 @@ fi
 
 # Test all available upgrades
 # TODO: protect via some variable ?
-utils/check_all_upgrades.sh \
-        `grep '^POSTGIS_' Version.config | cut -d= -f2 | paste -sd '.'`
-if [ "$?" != "0" ]; then
-  exit $?
+if [ "$CHECK_UPGRADE" = "1"]; then
+  utils/check_all_upgrades.sh \
+          `grep '^POSTGIS_' Version.config | cut -d= -f2 | paste -sd '.'`
+  if [ "$?" != "0" ]; then
+    exit $?
+  fi
 fi
