@@ -31,6 +31,7 @@
 #include "liblwgeom_internal.h"
 #include "lwgeom_log.h"
 
+#define out_stack_size 32
 
 /** Force Right-hand-rule on LWGEOM polygons **/
 void
@@ -774,42 +775,42 @@ lwgeom_segmentize2d(const LWGEOM *lwgeom, double dist)
 LWGEOM*
 lwgeom_force_2d(const LWGEOM *geom)
 {
-	return lwgeom_force_dims(geom, 0, 0);
+	return lwgeom_force_dims(geom, 0, 0, 0, 0);
 }
 
 LWGEOM*
-lwgeom_force_3dz(const LWGEOM *geom)
+lwgeom_force_3dz(const LWGEOM *geom, double zval)
 {
-	return lwgeom_force_dims(geom, 1, 0);
+	return lwgeom_force_dims(geom, 1, 0, zval, 0);
 }
 
 LWGEOM*
-lwgeom_force_3dm(const LWGEOM *geom)
+lwgeom_force_3dm(const LWGEOM *geom, double mval)
 {
-	return lwgeom_force_dims(geom, 0, 1);
+	return lwgeom_force_dims(geom, 0, 1, 0, mval);
 }
 
 LWGEOM*
-lwgeom_force_4d(const LWGEOM *geom)
+lwgeom_force_4d(const LWGEOM *geom, double zval, double mval)
 {
-	return lwgeom_force_dims(geom, 1, 1);
+	return lwgeom_force_dims(geom, 1, 1, zval, mval);
 }
 
 LWGEOM*
-lwgeom_force_dims(const LWGEOM *geom, int hasz, int hasm)
+lwgeom_force_dims(const LWGEOM *geom, int hasz, int hasm, double zval, double mval)
 {
 	if (!geom)
 		return NULL;
 	switch(geom->type)
 	{
 		case POINTTYPE:
-			return lwpoint_as_lwgeom(lwpoint_force_dims((LWPOINT*)geom, hasz, hasm));
+			return lwpoint_as_lwgeom(lwpoint_force_dims((LWPOINT*)geom, hasz, hasm, zval, mval));
 		case CIRCSTRINGTYPE:
 		case LINETYPE:
 		case TRIANGLETYPE:
-			return lwline_as_lwgeom(lwline_force_dims((LWLINE*)geom, hasz, hasm));
+			return lwline_as_lwgeom(lwline_force_dims((LWLINE*)geom, hasz, hasm, zval, mval));
 		case POLYGONTYPE:
-			return lwpoly_as_lwgeom(lwpoly_force_dims((LWPOLY*)geom, hasz, hasm));
+			return lwpoly_as_lwgeom(lwpoly_force_dims((LWPOLY*)geom, hasz, hasm, zval, mval));
 		case COMPOUNDTYPE:
 		case CURVEPOLYTYPE:
 		case MULTICURVETYPE:
@@ -820,7 +821,7 @@ lwgeom_force_dims(const LWGEOM *geom, int hasz, int hasm)
 		case POLYHEDRALSURFACETYPE:
 		case TINTYPE:
 		case COLLECTIONTYPE:
-			return lwcollection_as_lwgeom(lwcollection_force_dims((LWCOLLECTION*)geom, hasz, hasm));
+			return lwcollection_as_lwgeom(lwcollection_force_dims((LWCOLLECTION*)geom, hasz, hasm, zval, mval));
 		default:
 			lwerror("lwgeom_force_2d: unsupported geom type: %s", lwtype_name(geom->type));
 			return NULL;
@@ -1086,7 +1087,6 @@ lwgeom_is_collection(const LWGEOM *geom)
 int
 lwtype_is_collection(uint8_t type)
 {
-
 	switch (type)
 	{
 	case MULTIPOINTTYPE:
@@ -1606,7 +1606,6 @@ lwgeom_remove_repeated_points_in_place(LWGEOM *geom, double tolerance)
 		}
 		case MULTIPOINTTYPE:
 		{
-			static uint32_t out_stack_size = 32;
 			double tolsq = tolerance*tolerance;
 			uint32_t i, j, n = 0;
 			LWMPOINT *mpt = (LWMPOINT *)(geom);
@@ -2255,14 +2254,16 @@ static void lwgeom_subdivide_recursive(const LWGEOM *geom,
 				       uint8_t dimension,
 				       uint32_t maxvertices,
 				       uint32_t depth,
-				       LWCOLLECTION *col);
+				       LWCOLLECTION *col,
+				       double gridSize);
 
 static void
 lwgeom_subdivide_recursive(const LWGEOM *geom,
 			   uint8_t dimension,
 			   uint32_t maxvertices,
 			   uint32_t depth,
-			   LWCOLLECTION *col)
+			   LWCOLLECTION *col,
+			   double gridSize)
 {
 	const uint32_t maxdepth = 50;
 
@@ -2273,6 +2274,8 @@ lwgeom_subdivide_recursive(const LWGEOM *geom,
 	if (!box_in)
 		return;
 
+	LW_ON_INTERRUPT(return);
+
 	GBOX clip;
 	gbox_duplicate(box_in, &clip);
 	double width = clip.xmax - clip.xmin;
@@ -2308,7 +2311,7 @@ lwgeom_subdivide_recursive(const LWGEOM *geom,
 		/* Don't increment depth yet, since we aren't actually
 		 * subdividing geometries yet */
 		for (uint32_t i = 0; i < incol->ngeoms; i++ )
-			lwgeom_subdivide_recursive(incol->geoms[i], dimension, maxvertices, depth, col);
+			lwgeom_subdivide_recursive(incol->geoms[i], dimension, maxvertices, depth, col, gridSize);
 		return;
 	}
 
@@ -2412,31 +2415,31 @@ lwgeom_subdivide_recursive(const LWGEOM *geom,
 	{
 		LWGEOM *subbox = (LWGEOM *)lwpoly_construct_envelope(
 		    geom->srid, subbox1.xmin, subbox1.ymin, subbox1.xmax, subbox1.ymax);
-		LWGEOM *clipped = lwgeom_intersection(geom, subbox);
+		LWGEOM *clipped = lwgeom_intersection_prec(geom, subbox, gridSize);
 		lwgeom_simplify_in_place(clipped, 0.0, LW_TRUE);
 		lwgeom_free(subbox);
 		if (clipped && !lwgeom_is_empty(clipped))
 		{
-			lwgeom_subdivide_recursive(clipped, dimension, maxvertices, depth, col);
+			lwgeom_subdivide_recursive(clipped, dimension, maxvertices, depth, col, gridSize);
 			lwgeom_free(clipped);
 		}
 	}
 	{
 		LWGEOM *subbox = (LWGEOM *)lwpoly_construct_envelope(
 		    geom->srid, subbox2.xmin, subbox2.ymin, subbox2.xmax, subbox2.ymax);
-		LWGEOM *clipped = lwgeom_intersection(geom, subbox);
+		LWGEOM *clipped = lwgeom_intersection_prec(geom, subbox, gridSize);
 		lwgeom_simplify_in_place(clipped, 0.0, LW_TRUE);
 		lwgeom_free(subbox);
 		if (clipped && !lwgeom_is_empty(clipped))
 		{
-			lwgeom_subdivide_recursive(clipped, dimension, maxvertices, depth, col);
+			lwgeom_subdivide_recursive(clipped, dimension, maxvertices, depth, col, gridSize);
 			lwgeom_free(clipped);
 		}
 	}
 }
 
 LWCOLLECTION *
-lwgeom_subdivide(const LWGEOM *geom, uint32_t maxvertices)
+lwgeom_subdivide_prec(const LWGEOM *geom, uint32_t maxvertices, double gridSize)
 {
 	static uint32_t startdepth = 0;
 	static uint32_t minmaxvertices = 5;
@@ -2453,11 +2456,19 @@ lwgeom_subdivide(const LWGEOM *geom, uint32_t maxvertices)
 		lwerror("%s: cannot subdivide to fewer than %d vertices per output", __func__, minmaxvertices);
 	}
 
-	lwgeom_subdivide_recursive(geom, lwgeom_dimension(geom), maxvertices, startdepth, col);
+	lwgeom_subdivide_recursive(geom, lwgeom_dimension(geom), maxvertices, startdepth, col, gridSize);
 	lwgeom_set_srid((LWGEOM*)col, geom->srid);
 	return col;
 }
 
+LWCOLLECTION *
+lwgeom_subdivide(const LWGEOM *geom, uint32_t maxvertices)
+{
+	return lwgeom_subdivide_prec(geom, maxvertices, -1);
+}
+
+
+
 int
 lwgeom_is_trajectory(const LWGEOM *geom)
 {
