@@ -32,6 +32,8 @@
 #include <string.h>
 #include <stdio.h>
 
+#define maxprojlen  512
+#define spibufferlen 512
 
 
 /*
@@ -42,6 +44,9 @@
  */
 #define PROJ_BACKEND_HASH_SIZE 256
 
+/* Global to hold the Proj object cache */
+PROJSRSCache *PROJ_CACHE = NULL;
+
 
 /**
  * Utility structure to get many potential string representations
@@ -54,8 +59,9 @@ typedef struct {
 } PjStrs;
 
 /* Internal Cache API */
-static LWPROJ *AddToPROJSRSCache(PROJPortalCache *PROJCache, int32_t srid_from, int32_t srid_to);
-static void DeleteFromPROJSRSCache(PROJPortalCache *PROJCache, uint32_t position);
+static LWPROJ *
+AddToPROJSRSCache(PROJSRSCache *PROJCache, int32_t srid_from, int32_t srid_to);
+static void DeleteFromPROJSRSCache(PROJSRSCache *PROJCache, uint32_t position);
 
 static void
 PROJSRSDestroyPJ(void *projection)
@@ -82,215 +88,65 @@ PROJSRSDestroyPJ(void *projection)
 #endif
 }
 
-#if POSTGIS_PGSQL_VERSION < 96
 /**
- * Backend PROJ hash table
- *
- * This hash table stores a key/value pair of MemoryContext/PJ objects.
- * Whenever we create a PJ object using pj_init(), we create a separate
- * MemoryContext as a child context of the current executor context.
- * The MemoryContext/PJ object is stored in this hash table so
- * that when PROJSRSCacheDelete() is called during query cleanup, we can
- * lookup the PJ object based upon the MemoryContext parameter and hence
- * pj_free() it.
- */
-static HTAB *PJHash = NULL;
-
-typedef struct struct_PJHashEntry
-{
-	MemoryContext ProjectionContext;
-	LWPROJ *projection;
-} PJHashEntry;
-
-/* PJ Hash API */
-uint32 mcxt_ptr_hash(const void *key, Size keysize);
-
-static HTAB *CreatePJHash(void);
-static void DeletePJHashEntry(MemoryContext mcxt);
-static LWPROJ *GetPJHashEntry(MemoryContext mcxt);
-static void AddPJHashEntry(MemoryContext mcxt, LWPROJ *projection);
-
+* Destroy all the malloc'ed proj objects stored
+* in the PROJSRSCache, in case the containing
+* context is removed (shouldn't actually happen)
+*/
 static void
-PROJSRSCacheDelete(MemoryContext context)
+PROJSRSDestroyPortalCache(void *portalCache)
 {
-	/* Lookup the PJ pointer in the global hash table so we can free it */
-	LWPROJ *projection = GetPJHashEntry(context);
-
-	if (!projection)
-		elog(ERROR, "PROJSRSCacheDelete: Trying to delete non-existant projection object with MemoryContext key (%p)", (void *)context);
-
-	POSTGIS_DEBUGF(3, "deleting projection object (%p) with MemoryContext key (%p)", projection, context);
-	/* Free it */
-	PROJSRSDestroyPJ(projection);
-
-	/* Remove the hash entry as it is no longer needed */
-	DeletePJHashEntry(context);
-}
-
-static void
-PROJSRSCacheInit(MemoryContext context)
-{
-	/*
-	 * Do nothing as the cache is initialised when the transform()
-	 * function is first called
-	 */
-}
-
-static void
-PROJSRSCacheReset(MemoryContext context)
-{
-	/*
-	 * Do nothing, but we must supply a function since this call is mandatory according to tgl
-	 * (see postgis-devel archives July 2007)
-	 */
-}
-
-static bool
-PROJSRSCacheIsEmpty(MemoryContext context)
-{
-	/*
-	 * Always return false since this call is mandatory according to tgl
-	 * (see postgis-devel archives July 2007)
-	 */
-	return LW_FALSE;
-}
-
-static void
-PROJSRSCacheStats(MemoryContext context, int level)
-{
-	/*
-	 * Simple stats display function - we must supply a function since this call is mandatory according to tgl
-	 * (see postgis-devel archives July 2007)
-	 */
-
-	fprintf(stderr, "%s: PROJ context\n", context->name);
-}
-
-#ifdef MEMORY_CONTEXT_CHECKING
-static void
-PROJSRSCacheCheck(MemoryContext context)
-{
-	/*
-	 * Do nothing - stub required for when PostgreSQL is compiled
-	 * with MEMORY_CONTEXT_CHECKING defined
-	 */
-}
-#endif
-
-/* Memory context definition must match the current version of PostgreSQL */
-static MemoryContextMethods PROJSRSCacheContextMethods =
-{
-	NULL,
-	NULL,
-	NULL,
-	PROJSRSCacheInit,
-	PROJSRSCacheReset,
-	PROJSRSCacheDelete,
-	NULL,
-	PROJSRSCacheIsEmpty,
-	PROJSRSCacheStats
-#ifdef MEMORY_CONTEXT_CHECKING
-	,PROJSRSCacheCheck
-#endif
-};
-
-
-/*
- * PROJ PJ Hash Table functions
- */
-
-
-/**
- * A version of tag_hash - we specify this here as the implementation
- * has changed over the years....
- */
-
-uint32 mcxt_ptr_hash(const void *key, Size keysize)
-{
-	uint32 hashval;
-
-	hashval = DatumGetUInt32(hash_any(key, keysize));
-
-	return hashval;
-}
-
-
-static HTAB *CreatePJHash(void)
-{
-	HASHCTL ctl;
-
-	ctl.keysize = sizeof(MemoryContext);
-	ctl.entrysize = sizeof(PJHashEntry);
-	ctl.hash = mcxt_ptr_hash;
-
-	return hash_create("PostGIS PROJ Backend MemoryContext Hash", PROJ_BACKEND_HASH_SIZE, &ctl, (HASH_ELEM | HASH_FUNCTION));
-}
-
-static void
-AddPJHashEntry(MemoryContext mcxt, LWPROJ *projection)
-{
-	bool found;
-	void **key;
-	PJHashEntry *he;
-
-	/* The hash key is the MemoryContext pointer */
-	key = (void *)&mcxt;
-
-	he = (PJHashEntry*) hash_search(PJHash, key, HASH_ENTER, &found);
-	if (!found)
+	PROJSRSCache* cache = (PROJSRSCache*)portalCache;
+	for (uint32_t i = 0; i < cache->PROJSRSCacheCount; i++)
 	{
-		/* Insert the entry into the new hash element */
-		he->ProjectionContext = mcxt;
-		he->projection = projection;
-	}
-	else
-	{
-		elog(ERROR, "AddPJHashEntry: PROJ projection object already exists for this MemoryContext (%p)",
-		     (void *)mcxt);
+		if (cache->PROJSRSCache[i].projection)
+			PROJSRSDestroyPJ(cache->PROJSRSCache[i].projection);
 	}
 }
 
-static LWPROJ *
-GetPJHashEntry(MemoryContext mcxt)
+/**
+* Get the Proj cache entry from the global variable if one exists.
+* If it doesn't exist, make a new blank one and return it.
+*/
+PROJSRSCache *
+GetPROJSRSCache()
 {
-	void **key;
-	PJHashEntry *he;
+	PROJSRSCache* cache = PROJ_CACHE;
+	if (!cache)
+	{
+		/* Put proj cache in a child of the CacheContext */
+		MemoryContext context = AllocSetContextCreate(
+		    CacheMemoryContext,
+		    "Proj Context",
+		    ALLOCSET_SMALL_SIZES);
 
-	/* The hash key is the MemoryContext pointer */
-	key = (void *)&mcxt;
+		/* Allocate in the upper context */
+		cache = MemoryContextAllocZero(context, sizeof(PROJSRSCache));
 
-	/* Return the projection object from the hash */
-	he = (PJHashEntry *) hash_search(PJHash, key, HASH_FIND, NULL);
+		if (!cache)
+			elog(ERROR, "Unable to allocate space for PROJSRSCache in context %p", context);
 
-	return he ? he->projection : NULL;
+		cache->PROJSRSCacheCount = 0;
+		cache->PROJSRSCacheContext = context;
+
+		/* Use this to clean up PROJSRSCache in event of MemoryContext reset */
+		MemoryContextCallback* callback = MemoryContextAlloc(context, sizeof(MemoryContextCallback));
+		callback->func = PROJSRSDestroyPortalCache;
+		callback->arg = (void *)cache;
+		MemoryContextRegisterResetCallback(context, callback);
+
+		PROJ_CACHE = cache;
+	}
+	return cache;
 }
 
 
-static void DeletePJHashEntry(MemoryContext mcxt)
-{
-	void **key;
-	PJHashEntry *he;
-
-	/* The hash key is the MemoryContext pointer */
-	key = (void *)&mcxt;
-
-	/* Delete the projection object from the hash */
-	he = (PJHashEntry *) hash_search(PJHash, key, HASH_REMOVE, NULL);
-
-	if (!he)
-		elog(ERROR, "DeletePJHashEntry: There was an error removing the PROJ projection object from this MemoryContext (%p)", (void *)mcxt);
-	else
-		he->projection = NULL;
-}
-
-#endif /* POSTGIS_PGSQL_VERSION < 96 */
-
 /*****************************************************************************
  * Per-cache management functions
  */
 
 static LWPROJ *
-GetProjectionFromPROJCache(PROJPortalCache *cache, int32_t srid_from, int32_t srid_to)
+GetProjectionFromPROJCache(PROJSRSCache *cache, int32_t srid_from, int32_t srid_to)
 {
 	uint32_t i;
 	for (i = 0; i < cache->PROJSRSCacheCount; i++)
@@ -321,8 +177,6 @@ SPI_pstrdup(const char* str)
 static PjStrs
 GetProjStringsSPI(int32_t srid)
 {
-	const int maxprojlen = 512;
-	const int spibufferlen = 512;
 	int spi_result;
 	char proj_spi_buffer[spibufferlen];
 	PjStrs strs;
@@ -399,7 +253,6 @@ GetProjStringsSPI(int32_t srid)
 static PjStrs
 GetProjStrings(int32_t srid)
 {
-	const int maxprojlen = 512;
 	PjStrs strs;
 	memset(&strs, 0, sizeof(strs));
 
@@ -548,11 +401,8 @@ GetProj4String(int32_t srid)
  * which is the definition for the other half of the transformation.
  */
 static LWPROJ *
-AddToPROJSRSCache(PROJPortalCache *PROJCache, int32_t srid_from, int32_t srid_to)
+AddToPROJSRSCache(PROJSRSCache *PROJCache, int32_t srid_from, int32_t srid_to)
 {
-#if POSTGIS_PGSQL_VERSION < 96
-	MemoryContext PJMemoryContext;
-#endif
 	MemoryContext oldContext;
 
 	PjStrs from_strs, to_strs;
@@ -656,45 +506,18 @@ AddToPROJSRSCache(PROJPortalCache *PROJCache, int32_t srid_from, int32_t srid_to
 	pjstrs_pfree(&from_strs);
 	pjstrs_pfree(&to_strs);
 
-#if POSTGIS_PGSQL_VERSION < 96
-	/*
-	 * Now create a memory context for this projection and
-	 * store it in the backend hash
-	 */
-	PJMemoryContext = MemoryContextCreate(T_AllocSetContext, 8192,
-	                                      &PROJSRSCacheContextMethods,
-	                                      PROJCache->PROJSRSCacheContext,
-	                                      "PostGIS PROJ PJ Memory Context");
-
-	/* We register a new memory context to use it to delete the projection on exit */
-	if (!PJHash)
-		PJHash = CreatePJHash();
-
-	AddPJHashEntry(PJMemoryContext, projection);
-
-#else
-	/* We register a new callback to delete the projection on exit */
-	MemoryContextCallback *callback =
-	    MemoryContextAlloc(PROJCache->PROJSRSCacheContext, sizeof(MemoryContextCallback));
-	callback->func = PROJSRSDestroyPJ;
-	callback->arg = (void *)projection;
-	MemoryContextRegisterResetCallback(PROJCache->PROJSRSCacheContext, callback);
-#endif
-
+	/* Store everything in new cache entry */
 	PROJCache->PROJSRSCache[cache_position].srid_from = srid_from;
 	PROJCache->PROJSRSCache[cache_position].srid_to = srid_to;
 	PROJCache->PROJSRSCache[cache_position].projection = projection;
 	PROJCache->PROJSRSCache[cache_position].hits = hits;
-#if POSTGIS_PGSQL_VERSION < 96
-	PROJCache->PROJSRSCache[cache_position].projection_mcxt = PJMemoryContext;
-#endif
 
 	MemoryContextSwitchTo(oldContext);
 	return projection;
 }
 
 static void
-DeleteFromPROJSRSCache(PROJPortalCache *PROJCache, uint32_t position)
+DeleteFromPROJSRSCache(PROJSRSCache *PROJCache, uint32_t position)
 {
 	POSTGIS_DEBUGF(3,
 		       "removing query cache entry with SRIDs %d => %d at index %u",
@@ -702,15 +525,10 @@ DeleteFromPROJSRSCache(PROJPortalCache *PROJCache, uint32_t position)
 		       PROJCache->PROJSRSCache[position].srid_to,
 		       position);
 
-#if POSTGIS_PGSQL_VERSION < 96
-	/* Deleting the memory context will free the PROJ objects */
-	MemoryContextDelete(PROJCache->PROJSRSCache[position].projection_mcxt);
-	PROJCache->PROJSRSCache[position].projection_mcxt = NULL;
-#else
 	/* Call PROJSRSDestroyPJ to free the PROJ objects memory now instead of
 	 * waiting for the parent memory context to exit */
 	PROJSRSDestroyPJ(PROJCache->PROJSRSCache[position].projection);
-#endif
+
 	PROJCache->PROJSRSCache[position].projection = NULL;
 	PROJCache->PROJSRSCache[position].srid_from = SRID_UNKNOWN;
 	PROJCache->PROJSRSCache[position].srid_to = SRID_UNKNOWN;
@@ -718,18 +536,14 @@ DeleteFromPROJSRSCache(PROJPortalCache *PROJCache, uint32_t position)
 
 
 int
-GetPJUsingFCInfo(FunctionCallInfo fcinfo, int32_t srid_from, int32_t srid_to, LWPROJ **pj)
+GetLWPROJ(int32_t srid_from, int32_t srid_to, LWPROJ **pj)
 {
-	PROJPortalCache *proj_cache = NULL;
-
-	/* Look up the spatial_ref_sys schema if we haven't already */
-	postgis_initialize_cache(fcinfo);
-
 	/* get or initialize the cache for this round */
-	proj_cache = GetPROJSRSCache(fcinfo);
+	PROJSRSCache* proj_cache = GetPROJSRSCache();
 	if (!proj_cache)
 		return LW_FAILURE;
 
+	postgis_initialize_cache();
 	/* Add the output srid to the cache if it's not already there */
 	*pj = GetProjectionFromPROJCache(proj_cache, srid_from, srid_to);
 	if (*pj == NULL)
@@ -751,21 +565,21 @@ proj_pj_is_latlong(const LWPROJ *pj)
 }
 
 static int
-srid_is_latlong(FunctionCallInfo fcinfo, int32_t srid)
+srid_is_latlong(int32_t srid)
 {
 	LWPROJ *pj;
-	if ( GetPJUsingFCInfo(fcinfo, srid, srid, &pj) == LW_FAILURE)
+	if ( GetLWPROJ(srid, srid, &pj) == LW_FAILURE)
 		return LW_FALSE;
 	return proj_pj_is_latlong(pj);
 }
 
 void
-srid_check_latlong(FunctionCallInfo fcinfo, int32_t srid)
+srid_check_latlong(int32_t srid)
 {
 	if (srid == SRID_DEFAULT || srid == SRID_UNKNOWN)
 		return;
 
-	if (srid_is_latlong(fcinfo, srid))
+	if (srid_is_latlong(srid))
 		return;
 
 	ereport(ERROR, (
@@ -774,7 +588,7 @@ srid_check_latlong(FunctionCallInfo fcinfo, int32_t srid)
 }
 
 srs_precision
-srid_axis_precision(FunctionCallInfo fcinfo, int32_t srid, int precision)
+srid_axis_precision(int32_t srid, int precision)
 {
 	srs_precision sp;
 	sp.precision_xy = precision;
@@ -784,7 +598,7 @@ srid_axis_precision(FunctionCallInfo fcinfo, int32_t srid, int precision)
 	if ( srid == SRID_UNKNOWN )
 		return sp;
 
-	if ( srid_is_latlong(fcinfo, srid) )
+	if ( srid_is_latlong(srid) )
 	{
 		sp.precision_xy += 5;
 		return sp;
@@ -794,14 +608,14 @@ srid_axis_precision(FunctionCallInfo fcinfo, int32_t srid, int precision)
 }
 
 int
-spheroid_init_from_srid(FunctionCallInfo fcinfo, int32_t srid, SPHEROID *s)
+spheroid_init_from_srid(int32_t srid, SPHEROID *s)
 {
 	LWPROJ *pj;
 #if POSTGIS_PROJ_VERSION >= 48 && POSTGIS_PROJ_VERSION < 60
 	double major_axis, minor_axis, eccentricity_squared;
 #endif
 
-	if ( GetPJUsingFCInfo(fcinfo, srid, srid, &pj) == LW_FAILURE)
+	if ( GetLWPROJ(srid, srid, &pj) == LW_FAILURE)
 		return LW_FAILURE;
 
 #if POSTGIS_PROJ_VERSION >= 60
