@@ -145,8 +145,9 @@ Datum _postgis_gserialized_joinsel(PG_FUNCTION_ARGS);
 Datum _postgis_gserialized_stats(PG_FUNCTION_ARGS);
 
 /* Local prototypes */
-static Oid table_get_spatial_index(Oid tbl_oid, text *col, int *key_type);
-static GBOX * spatial_index_read_extent(Oid idx_oid, int key_type);
+static Oid table_get_spatial_index(Oid tbl_oid, text *col, int *key_type, int *att_num);
+static GBOX * spatial_index_read_extent(Oid idx_oid, int key_type, int att_num);
+
 
 /* Other prototypes */
 float8 gserialized_joinsel_internal(PlannerInfo *root, List *args, JoinType jointype, int mode);
@@ -726,6 +727,8 @@ nd_box_ratio(const ND_BOX *b1, const ND_BOX *b2, int ndims)
 	double vol2 = 1.0;
 	double vol1 = 1.0;
 
+	(void)vol1;
+
 	for ( d = 0 ; d < ndims; d++ )
 	{
 		if ( b1->max[d] <= b2->min[d] || b1->min[d] >= b2->max[d] )
@@ -1413,6 +1416,8 @@ compute_gserialized_stats_mode(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfu
 	int stats_slot;                     /* What slot is this data going into? (2D vs ND) */
 	int stats_kind;                     /* And this is what? (2D vs ND) */
 
+	(void)total_sample_volume;
+
 	/* Initialize sum and stddev */
 	nd_box_init(&sum);
 	nd_box_init(&stddev);
@@ -2281,7 +2286,7 @@ gserialized_sel_internal(PlannerInfo *root, List *args, int varRelid, int mode)
 	nd_stats = pg_nd_stats_from_tuple(vardata.statsTuple, mode);
 	ReleaseVariableStats(vardata);
 	selectivity = estimate_selectivity(&search_box, nd_stats, mode);
-	pfree(nd_stats);
+	if (nd_stats) pfree(nd_stats);
 	return selectivity;
 }
 
@@ -2312,11 +2317,12 @@ Datum gserialized_estimated_extent(PG_FUNCTION_ARGS)
 	char *tbl = NULL;
 	text *col = NULL;
 	char *nsp_tbl = NULL;
-	Oid tbl_oid, idx_oid;
+	Oid tbl_oid, idx_oid = 0;
 	ND_STATS *nd_stats;
 	GBOX *gbox = NULL;
 	bool only_parent = false;
-	int key_type;
+	int key_type, att_num;
+	size_t sz;
 
 	/* We need to initialize the internal cache to access it later via postgis_oid() */
 	postgis_initialize_cache();
@@ -2327,8 +2333,9 @@ Datum gserialized_estimated_extent(PG_FUNCTION_ARGS)
 		tbl = text_to_cstring(PG_GETARG_TEXT_P(1));
 		col = PG_GETARG_TEXT_P(2);
 		only_parent = PG_GETARG_BOOL(3);
-		nsp_tbl = palloc(strlen(nsp) + strlen(tbl) + 6);
-		sprintf(nsp_tbl, "\"%s\".\"%s\"", nsp, tbl);
+		sz = strlen(nsp) + strlen(tbl) + 6;
+		nsp_tbl = palloc(sz);
+		snprintf(nsp_tbl, sz, "\"%s\".\"%s\"", nsp, tbl);
 		tbl_oid = DatumGetObjectId(DirectFunctionCall1(regclassin, CStringGetDatum(nsp_tbl)));
 		pfree(nsp_tbl);
 	}
@@ -2337,8 +2344,9 @@ Datum gserialized_estimated_extent(PG_FUNCTION_ARGS)
 		nsp = text_to_cstring(PG_GETARG_TEXT_P(0));
 		tbl = text_to_cstring(PG_GETARG_TEXT_P(1));
 		col = PG_GETARG_TEXT_P(2);
-		nsp_tbl = palloc(strlen(nsp) + strlen(tbl) + 6);
-		sprintf(nsp_tbl, "\"%s\".\"%s\"", nsp, tbl);
+		sz = strlen(nsp) + strlen(tbl) + 6;
+		nsp_tbl = palloc(sz);
+		snprintf(nsp_tbl, sz, "\"%s\".\"%s\"", nsp, tbl);
 		tbl_oid = DatumGetObjectId(DirectFunctionCall1(regclassin, CStringGetDatum(nsp_tbl)));
 		pfree(nsp_tbl);
 	}
@@ -2346,8 +2354,9 @@ Datum gserialized_estimated_extent(PG_FUNCTION_ARGS)
 	{
 		tbl = text_to_cstring(PG_GETARG_TEXT_P(0));
 		col = PG_GETARG_TEXT_P(1);
-		nsp_tbl = palloc(strlen(tbl) + 3);
-		sprintf(nsp_tbl, "\"%s\"", tbl);
+		sz = strlen(tbl) + 3;
+		nsp_tbl = palloc(sz);
+		snprintf(nsp_tbl, sz, "\"%s\"", tbl);
 		tbl_oid = DatumGetObjectId(DirectFunctionCall1(regclassin, CStringGetDatum(nsp_tbl)));
 		pfree(nsp_tbl);
 	}
@@ -2357,17 +2366,22 @@ Datum gserialized_estimated_extent(PG_FUNCTION_ARGS)
 		PG_RETURN_NULL();
 	}
 
-#if 1
 	/* Read the extent from the head of the spatial index, if there is one */
-	idx_oid = table_get_spatial_index(tbl_oid, col, &key_type);
-	if (!idx_oid)
-		POSTGIS_DEBUGF(2, "index for \"%s.%s\" does not exist", tbl, text_to_cstring(col));
-	gbox = spatial_index_read_extent(idx_oid, key_type);
-#endif
 
-	/* Fall back to reading the stats, if no index answer */
-	if (!gbox)
+	idx_oid = table_get_spatial_index(tbl_oid, col, &key_type, &att_num);
+	if (idx_oid)
 	{
+		/* TODO: how about only_parent ? */
+		gbox = spatial_index_read_extent(idx_oid, key_type, att_num);
+		POSTGIS_DEBUGF(2, "index for \"%s.%s\" exists, reading gbox from there", tbl, text_to_cstring(col));
+		if ( ! gbox ) PG_RETURN_NULL();
+	}
+	else
+	{
+		POSTGIS_DEBUGF(2, "index for \"%s.%s\" does not exist", tbl, text_to_cstring(col));
+
+		/* Fall back to reading the stats, if no index is found */
+
 		/* Estimated extent only returns 2D bounds, so use mode 2 */
 		nd_stats = pg_get_nd_stats_by_name(tbl_oid, col, 2, only_parent);
 
@@ -2424,7 +2438,7 @@ Datum geometry_estimated_extent(PG_FUNCTION_ARGS)
 /************************************************************************/
 
 static Oid
-table_get_spatial_index(Oid tbl_oid, text *col, int *key_type)
+table_get_spatial_index(Oid tbl_oid, text *col, int *key_type, int *att_num)
 {
 	Relation tbl_rel;
 	ListCell *lc;
@@ -2463,6 +2477,7 @@ table_get_spatial_index(Oid tbl_oid, text *col, int *key_type)
 		{
 			Form_pg_attribute att;
 			Oid atttypid;
+			int attnum;
 			/* Is the index on the column name we are looking for? */
 			HeapTuple att_tup = SearchSysCache2(ATTNAME,
 			                                    ObjectIdGetDatum(idx_oid),
@@ -2472,6 +2487,7 @@ table_get_spatial_index(Oid tbl_oid, text *col, int *key_type)
 
 			att = (Form_pg_attribute) GETSTRUCT(att_tup);
 			atttypid = att->atttypid;
+			attnum = att->attnum;
 			ReleaseSysCache(att_tup);
 
 			/* Is the column actually spatial? */
@@ -2479,6 +2495,8 @@ table_get_spatial_index(Oid tbl_oid, text *col, int *key_type)
 			{
 				/* Save result, clean up, and break out */
 				result = idx_oid;
+				if (att_num)
+					*att_num = attnum;
 				if (key_type)
 					*key_type = (atttypid == b2d_oid ? STATISTIC_KIND_2D : STATISTIC_KIND_ND);
 				break;
@@ -2489,7 +2507,7 @@ table_get_spatial_index(Oid tbl_oid, text *col, int *key_type)
 }
 
 static GBOX *
-spatial_index_read_extent(Oid idx_oid, int key_type)
+spatial_index_read_extent(Oid idx_oid, int key_type, int att_num)
 {
 	BOX2DF *bounds_2df = NULL;
 	GIDX *bounds_gidx = NULL;
@@ -2522,7 +2540,7 @@ spatial_index_read_extent(Oid idx_oid, int key_type)
 		if (!GistTupleIsInvalid(ituple))
 		{
 			bool isnull;
-			Datum idx_attr = index_getattr(ituple, 1, idx_rel->rd_att, &isnull);
+			Datum idx_attr = index_getattr(ituple, att_num, idx_rel->rd_att, &isnull);
 			if (!isnull)
 			{
 				if (key_type == STATISTIC_KIND_2D)
@@ -2581,6 +2599,7 @@ Datum _postgis_gserialized_index_extent(PG_FUNCTION_ARGS)
 {
 	GBOX *gbox = NULL;
 	int key_type;
+	int att_num;
 	Oid tbl_oid = PG_GETARG_DATUM(0);
 	text *col = PG_GETARG_TEXT_P(1);
 	Oid idx_oid;
@@ -2591,11 +2610,11 @@ Datum _postgis_gserialized_index_extent(PG_FUNCTION_ARGS)
 	/* We need to initialize the internal cache to access it later via postgis_oid() */
 	postgis_initialize_cache();
 
-	idx_oid = table_get_spatial_index(tbl_oid, col, &key_type);
+	idx_oid = table_get_spatial_index(tbl_oid, col, &key_type, &att_num);
 	if (!idx_oid)
 		PG_RETURN_NULL();
 
-	gbox = spatial_index_read_extent(idx_oid, key_type);
+	gbox = spatial_index_read_extent(idx_oid, key_type, att_num);
 	if (!gbox)
 		PG_RETURN_NULL();
 	else
