@@ -1268,10 +1268,10 @@ rt_raster_gdal_polygonize(
 			RASTER_DEBUGF(4, "LWGEOM wkt = %s", wkt);
 			rtdealloc(wkt);
 
-			size_t lwwkbsize = 0;
-			uint8_t *lwwkb = lwgeom_to_wkb(lwgeom, WKB_ISO | WKB_NDR, &lwwkbsize);
+			lwvarlena_t *lwwkb = lwgeom_to_wkb_varlena(lwgeom, WKB_ISO | WKB_NDR);
+			size_t lwwkbsize = LWSIZE_GET(lwwkb->size) - LWVARHDRSZ;
 			if (lwwkbsize) {
-				d_print_binary_hex("LWGEOM wkb", lwwkb, lwwkbsize);
+				d_print_binary_hex("LWGEOM wkb", (const uint8_t *)lwwkb->data, lwwkbsize);
 				rtdealloc(lwwkb);
 			}
 		}
