@@ -278,6 +278,7 @@ static void
 rtpg_assignHookGDALEnabledDrivers(const char *enabled_drivers, void *extra) {
 	int enable_all = 0;
 	int disable_all = 0;
+	int vsicurl = 0;
 
 	char **enabled_drivers_array = NULL;
 	uint32_t enabled_drivers_count = 0;
@@ -294,6 +295,8 @@ rtpg_assignHookGDALEnabledDrivers(const char *enabled_drivers, void *extra) {
 	if (enabled_drivers == NULL)
 		return;
 
+	elog(DEBUG4, "Enabling GDAL drivers: %s", enabled_drivers);
+
 	/* destroy the driver manager */
 	/* this is the only way to ensure GDAL_SKIP is recognized */
 	GDALDestroyDriverManager();
@@ -325,6 +328,14 @@ rtpg_assignHookGDALEnabledDrivers(const char *enabled_drivers, void *extra) {
 			}
 		}
 	}
+	else if (strstr(enabled_drivers, GDAL_VSICURL) != NULL) {
+		for (i = 0; i < enabled_drivers_count; i++) {
+			if (strstr(enabled_drivers_array[i], GDAL_VSICURL) != NULL) {
+				enabled_drivers_found[i] = TRUE;
+				vsicurl = 1;
+			}
+		}
+	}
 
 	if (!enable_all) {
 		int found = 0;
@@ -393,6 +404,9 @@ rtpg_assignHookGDALEnabledDrivers(const char *enabled_drivers, void *extra) {
 			elog(WARNING, "Unknown GDAL driver: %s", enabled_drivers_array[i]);
 	}
 
+	if (vsicurl)
+		elog(WARNING, "%s set.", GDAL_VSICURL);
+
 	/* destroy the driver manager */
 	/* this is the only way to ensure GDAL_SKIP is recognized */
 	GDALDestroyDriverManager();
