@@ -2,7 +2,7 @@
 set client_min_messages to ERROR;
 
 -- Import city_data
-\i load_topology.sql
+\i ../load_topology.sql
 
 SELECT topology.ST_ModEdgeHeal('city_data', 1, null);
 SELECT topology.ST_ModEdgeHeal('city_data', null, 1);
@@ -94,7 +94,7 @@ SELECT r.topogeo_id, r.element_id
   FROM t.relation r, t.f f WHERE
   r.layer_id = layer_id(f.g) AND r.topogeo_id = id(f.g)
   AND r.topogeo_id in (2,3)
-  ORDER BY r.layer_id, r.topogeo_id;
+  ORDER BY r.layer_id, r.topogeo_id, r.element_id;
 
 -- This is fine, but will have to tweak definition of
 -- 'F+E3-E4' and 'F-E3+E4'
@@ -107,7 +107,7 @@ SELECT r.topogeo_id, r.element_id
   FROM t.relation r, t.f f WHERE
   r.layer_id = layer_id(f.g) AND r.topogeo_id = id(f.g)
   AND r.topogeo_id in (2,3)
-  ORDER BY r.layer_id, r.topogeo_id;
+  ORDER BY r.layer_id, r.topogeo_id, r.element_id;
 
 SELECT topology.DropTopology('t');
 
@@ -179,6 +179,21 @@ SELECT '#1998.-', topology.DropTopology('t1998');
 -------------------------------------------------------------------------
 -------------------------------------------------------------------------
 
+-- Crash case
+-- See https://trac.osgeo.org/postgis/ticket/5947
+
+BEGIN;
+SELECT NULL FROM createTopology('t5947');
+SELECT NULL FROM TopoGeo_addLinestring('t5947', 'LINESTRING(10 0, 0 0)');
+SELECT NULL FROM TopoGeo_addLinestring('t5947', 'LINESTRING(20 0, 10 0)');
+UPDATE t5947.edge_data SET geom = 'LINESTRING EMPTY' WHERE edge_id = 2;
+SELECT 't5947', ST_ModEdgeHeal('t5947', 1, 2);
+ROLLBACK;
+
+-------------------------------------------------------------------------
+-------------------------------------------------------------------------
+-------------------------------------------------------------------------
+
 -- TODO: test registered but unexistent topology
 -- TODO: test registered but corrupted topology
 --       (missing node, edge, relation...)
